import datetime
from django.db import models
from django.utils import timezone
from django.core.exceptions import ValidationError

# Create your models here.
class FileField(models.FileField):
    def __init__(self, *args, **kwargs):
        self.max_upload_size = kwargs.pop("max_upload_size", 1024*1024)
        super().__init__(*args, **kwargs)
        
    def upload_to(self, instance, filename):
        file = instance.file
        if file.size > self.max_upload_size:
            raise ValidationError("File size must be less than 1 MB")

class Role(models.Model):
    role = models.CharField(max_length=64)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    def __str__(self):
        return self.role
    
    
    

class Department(models.Model):
    department = models.CharField(max_length=64)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    def __str__(self):
        return self.department
    
    
    
class Designation(models.Model):
    designation = models.CharField(max_length=64)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    def __str__(self):
        return self.designation
    
    
class Staff(models.Model):
    gender_choices = (('Male', 'Male'), ('Female', 'Female'))
    staff_id = models.CharField(max_length=12, unique=True)
    role = models.ForeignKey(Role, on_delete=models.PROTECT, null=True, blank=True)
    department = models.ForeignKey(Department, on_delete=models.PROTECT, null=True, blank=True)
    designation = models.ForeignKey(Designation, on_delete=models.PROTECT, null=True, blank=True)
    name = models.CharField(max_length=65)
    gender = models.CharField(max_length=8, choices=gender_choices)
    dob = models.DateField()
    email = models.EmailField(null=True, blank=True)
    contact = models.CharField(max_length=65)
    bank_account_number = models.CharField(max_length=20, null=True, blank=True)
    # account_name = models.CharField(max_length=65, null=True, blank=True)
    bank_name = models.CharField(max_length=65, null=True, )
    bank_branch = models.CharField(max_length=65, null=True, blank=True)
    address = models.TextField("Address Line", max_length=200 ,blank=True, null=True)
    # image = models.ImageField(upload_to='uploads/', null=True, blank=True)
    basic_salary = models.DecimalField(decimal_places=2, max_digits=10, default=0.00)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)

    def __str__(self):
        return f'{self.name}' 
    

class Payroll(models.Model):
    YEAR_CHOICES = [(r,r) for r in range(datetime.date.today().year-1, datetime.date.today().year+1)]
    MONTH_CHOICES = (
        ('January', 'January'),
        ('February', 'February'),
        ('March', 'March'),
        ('April', 'April'),
        ('May', 'May'),
        ('June', 'June'),
        ('July', 'July'),
        ('August', 'August'),
        ('September', 'September'),
        ('October', 'October'),
        ('November', 'November'),
        ('December', 'December'),
    )
    
    staff = models.ForeignKey(Staff, on_delete=models.PROTECT)
    basic_salary = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    #house_allowance = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    #transport_allowance = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    #medical_allowance = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    #overtime = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    #gross_salary = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    tax = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    #pension = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    month = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    year = models.IntegerField(choices=YEAR_CHOICES, default=datetime.datetime.now().year, null=True, blank=True)
    net_salary = models.DecimalField(decimal_places=2, max_digits=10, null=True, blank=True)
    rent = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    penalty = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    advance_salary = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    deduction = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    bonus = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    allowance = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    total_deductions = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    net_salary = models.CharField(max_length=24, choices=MONTH_CHOICES, null=True, blank=True)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    def __str__(self):
        return f'{self.staff.name} {self.basic_salary}'
    
    
class Deductions(models.Model):
    payroll = models.ForeignKey(Payroll, on_delete=models.PROTECT)
    type = models.CharField(max_length=24)
    amount = models.DecimalField(decimal_places=2, max_digits=10)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    def __str__(self):
        return f'{self.payroll.staff}'
    
    
    
